@@ -1,10 +1,23 @@
-#include <ATen/ATen.h>
+#define TORCH_ASSERT_ONLY_METHOD_OPERATORS
+#include <ATen/core/Tensor.h>
 #include <ATen/Parallel.h>
-#include <ATen/native/quantized/cpu/embedding_packed_params.h>
+#include <ATen/native/quantized/cpu/EmbeddingPackedParams.h>
 #include <ATen/native/quantized/cpu/fbgemm_utils.h>
+#include <ATen/native/quantized/cpu/qembeddingbag.h>
+#include <c10/util/irange.h>
 #include <torch/library.h>
 
-torch::class_<EmbeddingPackedParamsBase> register_embedding_params();
+#ifndef AT_PER_OPERATOR_HEADERS
+#include <ATen/Functions.h>
+#include <ATen/NativeFunctions.h>
+#else
+#include <ATen/ops/_empty_per_channel_affine_quantized.h>
+#include <ATen/ops/empty.h>
+#include <ATen/ops/from_blob.h>
+#include <ATen/ops/resize_native.h>
+#endif
+
+int register_embedding_params();
 
 at::Tensor PackedEmbeddingBagWeight::unpack() {
   auto packed_weight = packed_w;
@@ -13,7 +26,7 @@ at::Tensor PackedEmbeddingBagWeight::unpack() {
   if (bit_rate_ == 8 || bit_rate_ == 4) {
     const auto input_rows = packed_weight.size(0);
     const auto input_columns = packed_weight.size(1);
-    int scale_bias_bytes;
+    int scale_bias_bytes = 0;
     const auto num_elem_per_byte = 8 / bit_rate_;
     if (bit_rate_ == 8) {
       // The last 2 values are used to store the FP32 scale and zero_point
@@ -23,7 +36,7 @@ at::Tensor PackedEmbeddingBagWeight::unpack() {
       scale_bias_bytes = 4;
     }
 
-    const auto* input = packed_weight.data_ptr<uint8_t>();
+    const auto* input = packed_weight.const_data_ptr<uint8_t>();
     // Calculate the output shape, accounting for the last n bytes to be used
     // for scale/bias rest of the entries are packed depending on the bit_width.
     std::vector<int64_t> output_shape = {
@@ -37,7 +50,7 @@ at::Tensor PackedEmbeddingBagWeight::unpack() {
         w_zp.data(), w_zp.size(), device(c10::kCPU).dtype(c10::kFloat));
 
     auto output_columns = output_shape[1];
-    uint8_t* output_data;
+    uint8_t* output_data = nullptr;
 
     // Allocate output weight tensor based on the bit_width
     if (bit_rate_ == 8) {
@@ -65,15 +78,15 @@ at::Tensor PackedEmbeddingBagWeight::unpack() {
     // For sub-byte tensors this will copy the packed bytes over since the
     // sub_byte qtensors are expected to store data in packed format.
     at::parallel_for(0, input_rows, 1, [&](int32_t start_idx, int32_t end_idx) {
-      for (int64_t row = start_idx; row < end_idx; ++row) {
+      for (const auto row : c10::irange(start_idx, end_idx)) {
         const std::uint8_t* input_row = input + row * input_columns;
         uint8_t* output_row =
             output_data + row * output_columns / num_elem_per_byte;
 
-        for (std::size_t col = 0; col < output_columns / num_elem_per_byte;
-             ++col) {
+        // output_columns
+        for (const auto col : c10::irange(output_columns / num_elem_per_byte)) {
           output_row[col] = input_row[col];
-        } // output_columns
+        }
       }
     });
 
@@ -85,45 +98,54 @@ at::Tensor PackedEmbeddingBagWeight::unpack() {
   return weight_origin;
 }
 
-namespace at {
-namespace native {
-namespace {
-
-Tensor qembeddingbag_byte_unpack(const Tensor& packed_weight) {
-  const auto input_rows = packed_weight.size(0);
-  const auto input_columns = packed_weight.size(1);
+namespace at::native {
 
+Tensor& qembeddingbag_byte_unpack_out(Tensor& output, const Tensor& packed_weight) {
+  // The "last" dimension of an N-Dimensioned batch of embedding bags is
+  // quantization channel. E.g. for a 2D embedding bag, this has
+  // [ row, col ] dimensions, for batched of embedding bags, dimensions might be
+  // [ batch, row, col ].
+  //
+  // Python Batched Embedding Example:
+  // weights = torch.from_numpy((np.random.random_sample((
+  //          2, 10, 3)).squeeze() + 1).astype(np.float32))
+  // assert(weights.size() == torch.Size([2, 10, 3]))
+  // # NOTE: 8 bytes (columns) are added due to fp32 zero_point and scales
+  // packed_weights = torch.ops.quantized.embedding_bag_byte_prepack(weights)
+  // assert(packed_weights.size() == torch.Size([2, 10, 11]))
+  // unpacked_weights = torch.ops.quantized.embedding_bag_byte_unpack(packed_weights)
+  // assert(unpacked_weights.size() == torch.Size([2, 10, 3]))
+  const auto packed_weight_sizes = packed_weight.sizes();
+  const auto col_dim = packed_weight_sizes.size() - 1;
+  const int64_t input_rows = c10::size_to_dim_(col_dim, packed_weight_sizes);
+  const int32_t input_columns = packed_weight_sizes[col_dim];
   // The last 2 values are used to store the FP32 scale and zero_point values
   // per row.
-  int output_columns = input_columns - 2 * sizeof(float);
+  const int32_t output_columns = input_columns - 2 * sizeof(float);
+  const auto* input_data = packed_weight.const_data_ptr<uint8_t>();
 
-  const auto* input = packed_weight.data_ptr<uint8_t>();
-  std::vector<int64_t> output_shape = {input_rows, output_columns};
-  at::Tensor output = at::empty(
-      output_shape,
-      packed_weight.options().dtype(kFloat),
-      packed_weight.suggest_memory_format());
-  float* output_data = output.data_ptr<float>();
+  std::vector<int64_t> output_shape = packed_weight_sizes.vec();
+  output_shape[col_dim] = output_columns;
+  at::native::resize_(output, output_shape);
+  auto output_contig = output.expect_contiguous();
+  float* output_data = output_contig->data_ptr<float>();
 
 #ifdef USE_FBGEMM
-    at::parallel_for(
-      0, input_rows, 1, [&](int32_t start_idx, int32_t end_idx) {
-        for (int64_t row = start_idx; row < end_idx; ++row) {
-          fbgemm::Fused8BitRowwiseQuantizedSBFloatToFloat(
-            input + row * input_columns,
-            1,
-            input_columns,
-            output_data + row * output_columns);
-        }
-      });
+  at::parallel_for(0, input_rows, 1, [&](int64_t start_idx, int64_t end_idx) {
+    fbgemm::Fused8BitRowwiseQuantizedSBFloatToFloatOrHalf<float>(
+        input_data + start_idx * input_columns,
+        end_idx - start_idx,
+        input_columns,
+        output_data + start_idx * output_columns);
+  });
 #else
-  for (std::size_t row = 0; row < input_rows; ++row) {
-    const std::uint8_t* input_row = input + row * input_columns;
+  for (auto row : c10::irange(input_rows)) {
+    const std::uint8_t* input_row = input_data + row * input_columns;
     const float* input_row_scale_zp =
         reinterpret_cast<const float*>(input_row + output_columns);
     float* output_row = output_data + row * output_columns;
 
-    for (std::size_t col = 0; col < output_columns; ++col) {
+    for (auto col : c10::irange(output_columns)) {
       output_row[col] =
           input_row[col] * input_row_scale_zp[0] + input_row_scale_zp[1];
     } // output_columns
@@ -132,12 +154,37 @@ Tensor qembeddingbag_byte_unpack(const Tensor& packed_weight) {
   return output;
 }
 
+namespace {
+Tensor qembeddingbag_byte_unpack(const Tensor& packed_weight) {
+  at::Tensor output = at::empty(
+      {},
+      packed_weight.options().dtype(kFloat),
+      packed_weight.suggest_memory_format());
+  qembeddingbag_byte_unpack_out(output, packed_weight);
+  return output;
+}
+
+Tensor qembeddingbag_byte_unpack_meta(const Tensor& packed_weight) {
+  const auto packed_weight_sizes = packed_weight.sym_sizes();
+  const auto col_dim = packed_weight_sizes.size() - 1;
+  const auto input_columns = packed_weight_sizes[col_dim];
+  // The last 2 values are used to store the FP32 scale and zero_point values
+  // per row.
+  const auto output_columns = input_columns - 2 * sizeof(float);
+
+  auto output_shape = packed_weight_sizes.vec();
+  output_shape[col_dim] = output_columns;
+
+  at::SymDimVector output_shape_vec(output_shape);
+  return at::empty_symint(output_shape_vec, packed_weight.options().dtype(kFloat), packed_weight.suggest_memory_format());
+}
+
 Tensor _qembeddingbag_nbit_unpack_helper(
     const Tensor& packed_weight,
     int BIT_RATE) {
   const auto input_rows = packed_weight.size(0);
   const auto input_columns = packed_weight.size(1);
-  const auto* input_data = packed_weight.data_ptr<uint8_t>();
+  const auto* input_data = packed_weight.const_data_ptr<uint8_t>();
   int NUM_ELEM_PER_BYTE = 8 / BIT_RATE;
 
   // The last 4 bytes per row are two fp16 scale and zero_point.
@@ -153,19 +200,17 @@ Tensor _qembeddingbag_nbit_unpack_helper(
       packed_weight.suggest_memory_format());
   float* output_data = output.data_ptr<float>();
 #ifdef USE_FBGEMM
-    at::parallel_for(
-      0, input_rows, 1, [&](int32_t start_idx, int32_t end_idx) {
-        for (int64_t row = start_idx; row < end_idx; ++row) {
-          fbgemm::FusedNBitRowwiseQuantizedSBHalfToFloat(BIT_RATE,
-            input_data + row * input_columns,
-            1,
-            input_columns,
-            output_data + row * output_dimensions[1]);
-        }
-      });
+  at::parallel_for(0, input_rows, 1, [&](int64_t start_idx, int64_t end_idx) {
+    fbgemm::FusedNBitRowwiseQuantizedSBHalfToFloatOrHalf<float>(
+        BIT_RATE,
+        input_data + start_idx * input_columns,
+        end_idx - start_idx,
+        input_columns,
+        output_data + start_idx * output_dimensions[1]);
+  });
 #else
   auto output_columns = output_dimensions[1];
-  for (size_t row = 0; row < input_rows; ++row) {
+  for (auto row : c10::irange(input_rows)) {
     float* output_row = output_data + row * output_columns;
     const std::uint8_t* input_row = input_data + row * input_columns;
     const at::Half* input_row_scale_zp = reinterpret_cast<const at::Half*>(
@@ -174,7 +219,7 @@ Tensor _qembeddingbag_nbit_unpack_helper(
     float scale = input_row_scale_zp[0];
     float zero_point = input_row_scale_zp[1];
 
-    for (int col = 0; col < output_columns; ++col) {
+    for (const auto col : c10::irange(output_columns)) {
       std::uint8_t quantized = input_row[col / NUM_ELEM_PER_BYTE];
       quantized >>= (col % NUM_ELEM_PER_BYTE) * BIT_RATE;
       quantized &= (1 << BIT_RATE) - 1;
@@ -238,6 +283,11 @@ TORCH_LIBRARY_IMPL(quantized, CatchAll, m) {
       TORCH_FN(QEmbeddingUnpackWeights::run));
 }
 
+TORCH_LIBRARY_IMPL(quantized, Meta, m) {
+  m.impl(
+      "quantized::embedding_bag_byte_unpack",
+      qembeddingbag_byte_unpack_meta);
+}
+
 } // namespace
-} // namespace native
-} // namespace at
+} // namespace at::native
